package uk.ac.ox.tall.wcker;

import java.io.*;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;

import uk.ac.ox.tall.wcker.WckerStep;
import uk.ac.ox.tall.wcker.OrganisationPayLoad;

/**
 * LoadWizard class - loads the "wizard.wsi" file which describes 
 * a WCKER wizard, and makes an object for {@link Wizard} to
 * use creating the running wizard.
 *                   
 * @author BalchD
 * @author JohnstonD
 *  
 */


public class LoadWizard {
	Document cacheDocument;
	static NeedToKnow ntn;

	Document getDocument()
	{
		return cacheDocument;
	}

    	static boolean debug = WckerSystem.doDebug();

	/**
 	* we derive a list of <input> nodes from an <inputs> node
	*/
	static WckerInput[] node_to_inputs(Node n) // inputs node
	{
		// We come here with an "inputs" node 
		String[] path = { "input" };
		Node[] ns = Navigate.elements_by_path( n ,  path);
		
		WckerInput[] is = new WckerInput[ns.length];
		for(int i = 0; i < ns.length; ++i)
		{
			is[i] = new WckerInput(ns[i]);
		}
		return is;
	}
	/**
	* convert a <question> XML node to a WckerQuestion java object
	*/
	static WckerQuestion node_to_question(Node n) // QuestionNode
	{
		WckerQuestion q = new WckerQuestion();
		String[] na1 = { "prompt" };
		
		Node[] prompt_nodes   = Navigate.elements_by_path(n, na1);
		q.prompt       = prompt_nodes[0].getChildNodes().item(0).getNodeValue();


		String[] na2 = { "guidance", "plainText" };

		Node[] guidance_nodes = Navigate.elements_by_path(n, na2);
		q.guidance = guidance_nodes[0].getChildNodes().item(0).getNodeValue();
		if ( debug )
		{
			System.out.println("PROMPT GUIDANCE" + q.prompt + q.guidance );
		}

		String[] na3 = { "inputs" };
		Node[] input_nodes = Navigate.elements_by_path(n, na3); 

		q.inputs = node_to_inputs(input_nodes[0]);

		if ( debug )
		{
			System.out.println("node_to_question q.inputs.length " + q.inputs.length); 
		}

		return q;
	}

	/**
	* convert a <questionGroup> XML node to an array of WckerQuestion java objects
	*/

	static WckerQuestion[] node_to_questions(Node n) // Question Group Node
	{	
		
		String name_array[] = {"question"};
		
		Node[] qs_arc = Navigate.find_child_elements_with_name_in_array( n , name_array );
		Node[] qs = Navigate.elements_by_path( n, name_array );

		if ( debug )
		{
			System.out.println(" node_to_questions " + qs.length);
			Navigate.recursive_print(" node_to_questions " ,  n);
			System.out.println(" node_to_questions " + qs.length);
		}

		WckerQuestion[] wqs = new WckerQuestion[qs.length];
		for( int i = 0; i < qs.length; ++i)
		{
			wqs[i] =  node_to_question( qs[i] );
		}
		
		return wqs; // we want to find 3 questions in our initial test example
	}

	/**
	* convert a questionGroup node to a  WckerQuestionGroup object
	*
	*/
	static WckerQuestionGroup node_to_question_group(Node n)
	{
		WckerQuestionGroup wqg = new WckerQuestionGroup();
		
		
		wqg.id = n.getAttributes().getNamedItem("id").getNodeValue();
		wqg.questions = node_to_questions(n);
		return wqg;
	}

	/**
         * convert an XML document to an array of WckerQuestionGroup objects
	 */
	static WckerQuestionGroup[] document_to_question_groups(Document d)
	{
		
		String[] array = { "wizard" , "questions", "questionGroup" };

		Node[] ns = Navigate.elements_by_path(  d, array );
	
		
		if ( debug )
		{
			System.out.println("document_to_question_groups " + ns.length );
			Navigate.node_print(ns[0]);
		}

		WckerQuestionGroup[] w = new WckerQuestionGroup[ns.length] ;
		for(int i = 0; i < ns.length; ++i)	
		{
			 w[i] = node_to_question_group(ns[i]);
		}

		return w;
	}
	/**
	*	extract document title
	*/
	
	static String document_to_simple_tag_contents(Document d, String s)
	{
		NodeList nl = d.getElementsByTagName(s);	
		return nl.item(0).getChildNodes().item(0).getNodeValue();
	}
	static String document_to_description(Document d)
	{
		return document_to_simple_tag_contents(d, "dc:description");
	}
	static String document_to_title(Document d)
	{
		return document_to_simple_tag_contents(d, "dc:title");
/****
		NodeList nl = d.getElementsByTagName("dc:title");	
		
		int l     = nl.getLength();
		Node n    =  nl.item(0);
		String s  = n.toString();
		String s2 = n.getNodeValue();

		NodeList nl2 = n.getChildNodes();

		if (debug )
		{
			System.out.println("tree details " + l );
			System.out.println("tree details " + s2 );
			System.out.println("tree details " + nl2 );
			System.out.println("tree details START");

			for(int j=0;j<nl2.getLength();++j)
			{
				System.out.println( j + ")" +  nl2.item(j).toString()   +  nl2.item(j).getNodeValue());
			}
			System.out.println("tree details STOP");
		}
		
		return nl2.item(0).getNodeValue();
****/
	}
	/**
	 * recursively print step nodes that are hanging off a document
         */

	static void recursive_print(Document d)
	{
		NodeList nl = d.getElementsByTagName("step");
		int k;
		int i;
		
		
		int l     = nl.getLength();

		for(k=0; k < nl.getLength(); ++k)
		{
			Navigate.recursive_print( "" + k , nl.item(k));
		}
	}
	/**
	* extract contents of children plainText nodes
	* these will be concatinated if there is more than one match
	*/
	static String extract_plaintext(Node n)
	{
		Node[] nl = Navigate.find_child_elements_with_name(  n ,  "plainText" );
		String pt = "";
		for(int j=0; j < nl.length; ++j)
		{
			 pt += nl[j].getChildNodes().item(0).getNodeValue();
		}
		return pt;
	}
	static void applyHeirarchy(Node n)
	{
	}
	static void deal_with_question(Node q)
	{
		if ( debug )
		{
			System.out.println("Called deal_with_question");
		}

		Node[] prompt_a    = Navigate.find_child_elements_with_name(q,"prompt");
		Node[] guidance_a  = Navigate.find_child_elements_with_name(q,"guidance");
		Node[] inputs_a    = Navigate.find_child_elements_with_name(q,"inputs");

		Node prompt    = prompt_a[0];
		Node guidance  = guidance_a[0];
		Node inputs    = inputs_a[0];

		String p = prompt.getChildNodes().item(0).getNodeValue();
		if ( debug )
		{
			System.out.println("PROMPT " + p);
		}

		
		Node[] plaintext_a = Navigate.find_child_elements_with_name(guidance,"plainText");
		Node plaintext = plaintext_a[0];
		
		String g = plaintext.getChildNodes().item(0).getNodeValue();
		if ( debug )
		{
			System.out.println("GUIDANCE " + g);
		}
		
	}
	/**
	 * does what it says on the label
	 */
	static void deal_with_question_group(Node q)
	{
		Node[] c = Navigate.find_child_elements_with_name( q, "question");
		for(int i=0;i<c.length;++i)
		{
			deal_with_question(c[i]);
		}
		
		// find all questions foreach
			// extract prompt
			// extract quidance
			// inputs is type
	}
	static void applyItem(Node n)
	{
		Node[] c = Navigate.find_child_elements_with_name( n, "questionGroup");
		int len = c.length;
                /*
			if there is a ref get the main jobbo
		*/
		String question_group_name = c[0].getAttributes().getNamedItem("ref").getNodeValue();
		if ( debug )
		{
			System.out.println("applyItem " + len + question_group_name);
		}
		/*
			looking for "questions", "questionGroup with id="basic-details"
		*/
		String[] path = { "wizard", "questions", "questionGroup" };
		
		Node wanted = Navigate.element_by_path_and_attribute_value( n.getOwnerDocument(),  path, "id", "basic-details" );
		deal_with_question_group(wanted);
		if ( debug )
		{
			System.out.println("Found question groups with name basic-details");
		}
		
	}
	/** recursive descent */
	static void extract_apply(Node n)
	{
		Node[] nl = Navigate.find_child_elements_with_name(  n ,  "applyToHeirarchy" );
		Node[] nl2 = Navigate.find_child_elements_with_name(  n ,  "applyToItem" );
		if ( debug )
		{
			System.out.println("applyToHeirarchy: " +   nl.length + " applyToItem: " + nl2.length );
		}
		for(int i=0;i<nl.length;++i)
		{
			applyHeirarchy(nl[i]);
		}
		for(int i=0;i<nl2.length;++i)
		{
			applyItem(nl2[i]);
		}
	}
	/*
	* build up a WckerTree picking up particular elements and storing a particular attribute value
	*/
	static WckerTree structure_stuff_node(WckerTree parent, Node n, String element_name, String attribute_name)
	{
		NamedNodeMap nnm = n.getAttributes();
		Node att_node = nnm.getNamedItem(attribute_name);
		String label  = att_node.getNodeValue();
if (debug )
{
	System.out.print("365 structure_stuff_node " + " " + element_name + " " + attribute_name + "=" + label);
}
		//WckerTree t = new WckerTree(label);
		WckerTree t = 	new WckerTree
				(
					 parent,
					new StructurePayLoad
					(
						new String[] { label }
					)  
				);

		// but actually look for item OR itenPattern

		Node[] children = Navigate.find_child_elements_with_name(n,element_name);
		t.children = new WckerTree[children.length];

		for(int i=0;i < children.length; ++i)
		{	
			t.children[i] = structure_stuff_node(t,  children[i], element_name, attribute_name );
		}
		
		return t;
		
	}
	/*
	* build up a WckerTree picking up particular elements and storing two particular attribute values
	*/
//	static WckerTree structure_stuff_node2(Node n, String[] element_name_array, String attribute_name, String attribute_name2 )
	static WckerTree structure_stuff_node2(WckerTree parent, Node n, String[] element_name_array, String[] attribute_name_array )
	{
		//String label;
		//String type;
		String[] value = new String[attribute_name_array.length];

if ( debug )
{
	System.out.println("555 lookinf for " + attribute_name_array.length + " attibutes");
}

		for(int i = 0; i < attribute_name_array.length; ++i)
		{
			try
			{
				NamedNodeMap nnm = n.getAttributes();
				Node att_node = nnm.getNamedItem(attribute_name_array[i]);
				value[i]  = att_node.getNodeValue();
			}
			catch (Exception e)
			{
				value[i]  = "";
			}
if ( debug )
{
	System.out.println("555  lookinf for value[" + i + "] is  " + value[i]);
}
		}

		WckerTree t = new WckerTree(parent, new OrganisationPayLoad(value) );
/**
	System.out.println
	(
		"label_path = " + 
		(new NodeInfo(ntn, t)).label_path()
	);	
***/

		int dst;
		int i;

		Node[] children = Navigate.find_child_elements_with_name_in_array(n,element_name_array);

		for(dst = 0, i=0;i < children.length; ++i)
		{	
			WckerTree child = structure_stuff_node2( t, children[i], element_name_array, attribute_name_array ); 

			int count;
			try
			{
				NamedNodeMap nnm = children[i].getAttributes();
				Node att_node = nnm.getNamedItem("count");
				String label  = att_node.getNodeValue();
				count = new Integer(label).intValue();
			}
			catch (Exception e)
			{
				count = 1;
			}
			dst += count;
		}

		t.children = new WckerTree[dst];

		for(dst = 0,  i=0;i < children.length; ++i)
		{	
			WckerTree child = structure_stuff_node2( t, children[i], element_name_array, attribute_name_array  ); 

			int count;
			try
			{
				NamedNodeMap nnm = children[i].getAttributes();
				Node att_node = nnm.getNamedItem("count");
				String label  = att_node.getNodeValue();
				count = new Integer(label).intValue();
			}
			catch (Exception e)
			{
				count = 1;
			}
			for( int j = 0; j < count; ++j )
			{
				t.children[dst] = child.duplicate(t);
				if ( count > 1 )	
				{
					Object o = t.children[dst].payload;
					 Class c = o.getClass();
					String class_name = c.getName();
					if ( debug )
					{
						System.out.println("333 Class name is " + class_name );
					}

					((OrganisationPayLoad) t.children[dst].payload).nodename += " " + ( new Integer(j+1)).toString();

					//t.children[dst].nodename += " " + ( new Integer(j+1)).toString();
				}
				dst++;
			}
		}
		
		return t;
		
	}
	static WckerTree structure_stuff(Document d)
	{
		String[] top_terms 	= { "top" };
		StructurePayLoad spl 	= new StructurePayLoad( top_terms );
		WckerTree t 		= new WckerTree( (WckerTree) null, spl);

		String[] path = { "wizard", "init", "structureTerms", "structureItem" };

		Node si[]     = Navigate.elements_by_path(d, path);
		t.children = new WckerTree[si.length];

if (debug )
{
	System.out.print("365 structure_stuff length = " + si.length);
}

		for(int i=0; i < si.length; ++i)
		{
			t.children[i] = structure_stuff_node(t, si[i], "structureItem", "label");
		}
		return t;

	}
	static newPageTemplate document_to_template(Document d)
	{
		String[] path = { "wizard", "init", "newPageTemplate" };
		newPageTemplate ntp = new newPageTemplate();
		try
		{
				Node si[]     = Navigate.elements_by_path(d, path);
				NamedNodeMap nnm = si[0].getAttributes();
				Node att_node = nnm.getNamedItem("templatesrc");
				ntp.templatesrc  = att_node.getNodeValue();
		}
		catch (Exception e)
		{
				ntp.templatesrc  = "";
		}

		try
		{
				Node si[]     = Navigate.elements_by_path(d, path);
				NamedNodeMap nnm = si[0].getAttributes();
				Node att_node = nnm.getNamedItem("type");
				ntp.type  = att_node.getNodeValue();
		}
		catch (Exception e)
		{
				ntp.type  = "";
		}
				

		
		return ntp;
	}

	static String[] support_files_from_document(Document d) {
		String[] path = {"wizard", "init", "supportFiles", "copy"};
		
		try {
			Node copyElements[] = Navigate.elements_by_path(d, path);
			int numberElements = copyElements.length;
			String[] supportFiles = new String[numberElements];
			
			if(debug) {
				System.out.println("supportFiles copyElements[]: " + copyElements);
				System.out.println("supportFiles copyElements.length: " + copyElements.length);
				System.out.println("supportFiles copyElements[0]: " + copyElements[0]);
			}
			for (int i=0; i<numberElements; i++) {
				NamedNodeMap copyAttribs = copyElements[i].getAttributes();
				supportFiles[i] = copyAttribs.getNamedItem("source").getNodeValue();
			}
			return supportFiles;
		} catch (Exception e) {
			String[] supportFiles = null;
			return supportFiles;
		}
	}
	
	static WckerTree document_to_tree2(Document d)
	{
		
			

		WckerTree t = 
		new WckerTree
		( 
			(WckerTree) null,
			new OrganisationPayLoad
			(
				new String[]
				{
					"top","_top_type","","",""
				}
			)
		);
		String[] path = { "wizard", "init", "organization" };
		String[] element_name_array = { "item", "itemPattern" };

		Node si[]     = Navigate.elements_by_path(d, path);
		t.children = new WckerTree[si.length];
		if ( debug )
		{
			System.out.println("LYRA no of items is " + si.length);
		}
		String[] attribute_name_array = { "title", "type", "src", "templatesrc", "templatetype" };
		return structure_stuff_node2(t, si[0], element_name_array, attribute_name_array );
	}

	static WckerTree document_to_tree(Document d)
	{
		WckerTree t = structure_stuff(d);
		return t;
	}
	static String node_to_action(Node n)
	{
		String[][] paths = { { "exportSkeleton" }, { "importSkeleton" }, { "exportPackage" } };

		for(int i = 0; i < paths.length; ++i)	
		{
			Node[] ns = Navigate.elements_by_path(n, paths[i]);
			if ( ns.length > 0 )
			{
				return paths[i][0];
			}
		}
		return "";
	}
	static WckerStep node_to_step(Node n)
	{
		WckerStep ws = new WckerStep();
		ws.action = node_to_action(n);

		try 
		{
			String[] path = { "applyToHeirarchy", "questionGroup" };
			Node[] ns = Navigate.elements_by_path(n, path);
			ws.questionGroup_ref = ns[0].getAttributes().getNamedItem("ref").getNodeValue();
			ws.apply_name = "applyToHeirarchy";
		}
		catch (Exception e1)
		{
			try
			{
				String[] path = { "applyToItem", "questionGroup" };
				Node[] ns = Navigate.elements_by_path(n, path);
				ws.questionGroup_ref = ns[0].getAttributes().getNamedItem("ref").getNodeValue();			     
				ws.apply_name = "applyToItem";
			}
			catch (Exception e2)
			{
				ws.questionGroup_ref ="";
				ws.apply_name = "";
			}
		}

if ( debug )
{
	System.out.println("YYY READING IN |" + ws.questionGroup_ref + "|");
}


		String[] path = { "guidance", "plainText"};
		Node[] ns = Navigate.elements_by_path( n ,  path);

		try
		{
			ws.guidance = ns[0].getChildNodes().item(0).getNodeValue();
		}
		catch (Exception e)
		{
			ws.guidance = "";
		}


		extract_apply(n);

		ws.plaintext = extract_plaintext(n);

		if ( debug )
		{
			System.out.println("PLAINTEXT:"+extract_plaintext(n) );
		}

		try
		{
			String[] path2 = { "applyToHeirarchy", "guidance", "plainText"};
			Node[] ns2 = Navigate.elements_by_path( n ,  path2);
			ws.structure_guidance = ns2[0].getChildNodes().item(0).getNodeValue();
		}
		catch( Exception e)
		{
			ws.structure_guidance = "";
		}
		try
		{
			 String[] path3 =  {  "applyToHeirarchy", "prompt"};
			Node[] ns3 = Navigate.elements_by_path( n ,  path3 );
			ws.structure_prompt = ns3[0].getChildNodes().item(0).getNodeValue();
		}
		catch( Exception e)
		{
			ws.structure_prompt = "";
		}


		NamedNodeMap nnm = n.getAttributes();		// TITLE OF STEP
		Node r = nnm.getNamedItem("title");
		
		ws.title = r.getNodeValue() ;
		
  		return ws;
		
	}
	static WckerStep[] document_to_steps(Document d)
	{
		NodeList nl = d.getElementsByTagName("step");
		WckerStep[] wsa = new WckerStep[ nl.getLength() ];

		int l     = nl.getLength();
		String[] s = new String[l]; 
		String[] pta = new String[l]; 
		String[][] two = { s, pta };

		for(int k=0; k < nl.getLength(); ++k)		// FOR EACH STEP
		{
			wsa[k] = node_to_step( nl.item(k) );

		}
		
		return wsa;
	}
	/*
	* loads XML; converts to Document; converts in turn to NeedToKnow
	*/
	public LoadWizard(File wizardInstanceFile)
	{		

		cacheDocument = LoadXML.loadXml(wizardInstanceFile.toString());
		Document d = getDocument();
		if ( debug )
		{
			Navigate.RECURSIVE_print(d); // for debugging
		}
		ntn = new NeedToKnow(wizardInstanceFile.toString(), d);
		if ( debug )
		{
			System.out.println(ntn.toString());
		}

	}
	public static void main(File wizardInstanceFile)
	{
		LoadWizard tc = new LoadWizard(wizardInstanceFile);

	}
}
