package uk.ac.ox.tall.wcker;

import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.NamedNodeMap;

public class Navigate
{
    	static boolean debug = WckerSystem.doDebug();
	/**
	* convert a node to a string representation of its type
	*/
	static String node_type(Node n)
	{
		switch ( n.getNodeType() )
		{
		
			case Node.ATTRIBUTE_NODE:		return "ATTRIBUTE_NODE";
			case Node.CDATA_SECTION_NODE:		return "CDATA_SECTION_NODE";
			case Node.COMMENT_NODE:			return "COMMENT_NODE";
			case Node.DOCUMENT_FRAGMENT_NODE:	return "DOCUMENT_FRAGMENT_NODE";
			case Node.DOCUMENT_NODE:		return "DOCUMENT_NODE";
			case Node.DOCUMENT_TYPE_NODE:		return "DOCUMENT_TYPE_NODE";
			case Node.ELEMENT_NODE:			return "ELEMENT_NODE";
			case Node.ENTITY_NODE:			return "ENTITY_NODE";
			case Node.ENTITY_REFERENCE_NODE:	return "ENTITY_REFERENCE_NODE";
			case Node.NOTATION_NODE:		return "NOTATION_NODE";
			case Node.PROCESSING_INSTRUCTION_NODE:	return "PROCESSING_INSTRUCTION_NODE";
			default:				return "UNKNOWN NODE";
		}

	}
	/**
	*	print details about a Node to the screen
	*
	*/
	static void node_print( Node n)
	{
		System.out.println	
		( 
			"TOSTRING:" 	+ n.toString() 		+ "\n" +
			"GETNODENAME:" 	+ n.getNodeName() 	+ "\n" + 
			"GETNODEVALUE:"	+ n.getNodeValue() 	+ "\n" + 
			"GETNODETYPE:" 	+ node_type(n)		+ "\n" 
		);
	}
	/**
	 * recursively print a Node structure
         */

	static void recursive_print(String context, Node n)
	{
		int i;
		int na = 0;
		System.out.println( context + "----------------------------------------------------------");
		node_print(n);
		NamedNodeMap nnm = n.getAttributes();
		if ( nnm != null )
		{
			na = nnm.getLength();
			for(i=0;i < na; ++ i)
			{
				char A = ( new String("A") ).toCharArray()[0];
				byte[] b = ( new String("A") ).getBytes();
				int  I = ( (int) b[0] ) + i;
				b[0] = (byte) I;
				recursive_print( context + new String(b), nnm.item(i));
			}
			
		}
		NodeList nl2 = n.getChildNodes();
		int count = nl2.getLength();

		if ( count == 0 )
		{
			
		}
		else
		{
			for(i=0;i < nl2.getLength(); ++ i)
			{
				recursive_print(context + i, nl2.item(i));
			}
		}
	}
	/**
	 * recursively print a document
         */
	static void RECURSIVE_print(Document d)
	{
		NodeList nl = d.getChildNodes();

		for(int k=0; k < nl.getLength(); ++k)
		{
			recursive_print( "" + k , nl.item(k));
		}
	}
	/**
         * does exactly what it says on the label
	 */

	static Node[] find_child_elements_with_name( Node n , String name )
	{
		NodeList nl2 = n.getChildNodes();
		if ( debug )
		{
			System.out.println("NO STEP NODES:" + nl2.getLength() );
		}	
		int count = 0;
		for(int j=0; j < nl2.getLength(); ++j) // FIND NUMBER OF MATCHING NODES
		{
			if ( ( nl2.item(j).getNodeType() == Node.ELEMENT_NODE) && ( nl2.item(j).getNodeName() == name) )
			{
				count++;
			}	
		}	
		Node[] na2 = new Node[count];		// ALLOCATE STORAGE FOR THESE
		count = 0;
		for(int j=0; j < nl2.getLength(); ++j)  // FILL IN THE VALUES AGAIN
		{
			if ( ( nl2.item(j).getNodeType() == Node.ELEMENT_NODE) && ( nl2.item(j).getNodeName() == name) )
			{
				na2[count++] = nl2.item(j) ;
			}	
		}
		return na2;
	}
	/**
	* does exactly what it says on the label
	* we may be looking for more than one node type at the same time
	*
	*/
	static Node[] find_child_elements_with_name_in_array( Node n , String[] name )
	{
		NodeList nl2 = n.getChildNodes();
		if ( debug )
		{
			System.out.println("NO STEP NODES:" + nl2.getLength() );	
		}

		int count = 0;
		for(int j=0; j < nl2.getLength(); ++j) // LOOKING FOR PLAINTEXT
		{
			for(int k = 0; k < name.length; ++k)
			{
				if ( ( nl2.item(j).getNodeType() == Node.ELEMENT_NODE) && ( nl2.item(j).getNodeName() == name[k]) )
				{
					count++;
					break;
				}
			}	
		}
		Node[] na2 = new Node[count];
		count = 0;

		for(int j=0; j < nl2.getLength(); ++j) // LOOKING FOR PLAINTEXT
		{
			for(int k = 0; k < name.length; ++k)
			{
				if ( ( nl2.item(j).getNodeType() == Node.ELEMENT_NODE) && ( nl2.item(j).getNodeName() == name[k]) )
				{
					na2[count++] = nl2.item(j) ;
					break;
				}
			}	
		}
		return na2;
	}
	/** concatenate two arrays together */
	static Node[] node_add(Node[] a, Node[] b)
	{
		int la = a.length;
		int lb = b.length;
		Node[] c = new Node[la+lb];
		for(int i=0;i<la;++i)
		{
			c[i] = a[i];
		}
		for(int i=0;i<lb;++i)
		{
			c[i+la] = b[i];
		}
		return c;
	}
	/**
	* find XML elements by recursive path name
	*
	*/
	static Node[] elements_by_path( Node d, String[] path)
	{	
		Node[] inc = new Node[0];

		Node n0 = (Node) d;
		Node[] na2 = {};

		Node[] na = { n0 };
		
		for ( int i = 0; i < path.length; ++i)	
		{
			na2 = new Node[0];	// for next level down 
			for ( int j = 0; j < na.length; ++ j ) // for each input node
			{
				inc = find_child_elements_with_name( na[j], path[i]);
				if (debug)
				{
					System.out.println("element_by_path_and_attribute_value: element name = '" +  path[i] + "' no= " + inc.length);
				}
				na2 = node_add(na2,inc);	
			}
			na = na2;	// for next level down 
		}
		return na2;
	}
	

	/**
	* find XML elements by recursive path name and attribute value
	*
	*/
	
	static Node element_by_path_and_attribute_value( Document d, String[] path, String attribute_name, String attribute_value)
	{	

		Node[] na2 = elements_by_path(  d,  path);
	

		for( int i = 0; i < na2.length; ++i) // * for each question group
		{
			String question_group_name = na2[i].getAttributes().getNamedItem(attribute_name).getNodeValue();
			if ( debug )
			{
				System.out.println("question_group_name = " + question_group_name);
			}

			if ( question_group_name.equals(attribute_value) )
			{
				return na2[i];
			}
		}
			/**** all the "questions" s ***/
			/**** all the "questionGroup" s ***/

		System.err.println("haven't found expected element with aparticular attribute value");
		System.exit(-1);

		return null;
	}
	
}