package uk.ac.ox.tall.wcker;

import java.io.File;

import org.eclipse.swt.widgets.Tree;
import org.eclipse.swt.widgets.TreeItem;

import org.w3c.dom.Document;

import uk.ac.reload.editor.contentpackaging.xml.ContentPackage;

/**
 * NeedToKnow class - contains info from "TALL_short-course.wsi" in an application or
 *                    programmer friendly form. Basically, after the document is parsed
 *		      it gets translated into a "NeedToKnow" object, which is then
 *		      used to draw the GUI. Basically, the "in memory" form of the XML
 *                    is complex and confusing!
 * @author BalchD
 * @author JohnstonD
 *  
 */

public class NeedToKnow
{
	/* graphics stuff */
	Tree gui_tree;
	PreviousNext pn;
	TreeButtons tbs;
	
	// NeedToTell
	String filename;
	String course_title = "Looking at the world - rose tinted specs";
	// NeedToTell
    	boolean debug = WckerSystem.doDebug();

	public int stage = 3;			// which stage we are at - set to value 
						// to draw particular page for tall thin slice
	
	private File _initialDir; // The directory that the wizard's course is first created  
	private File _workingDir = null; // The directory within which the wizard's course is being worked on.
	
	public String title;
	public String description;
	public WckerStep[] steps;
	//public String[] plaintext;
	public WckerTree structure_terms;
	public newPageTemplate new_page_template;
	public String[] supportFiles;
	public WckerTree organization;
	
	public ContentPackage cp;
	public WckerQuestionGroup[] question_groups;
	final String pre_amble = 
"<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">"
+ "<html>\n"
+ "<head>\n"
+ "\n<style>\nbody{\nbackground:url(\"file:///D|/icons/progress_background.png\") no-repeat;\nmargin:0px;\nfont-family:verdana,sans-serif;\nfont-size:0.8em;\n}\nol\n{\nmargin-left:2em;\nfont-size:0.9em;\n}\nli \n{margin-left:2em;\npadding-top:0.2em;\npadding-bottom:0.2em;\n}\nh1\n{font-size:1em;\nbackground-color:#000080;\ncolor:#fff;\npadding-bottom:0.3em;\npadding-left:1em;\n}\nh2{\nfont-size:1em;color:#000080;\nmargin-top:0.5em;}\n p{font-size} .selected{font-weight:bold;}\ndt{font-weight:bold;margin-top:1em;}\ndd{margin-left:0.5em;font-size:0.9em;} h2, p, dl{margin-left:0.5em;}</style>"
+ "</head>\n"
+ "<body>\n";

final String post_amble = 
"</body>\n"
+ "</html>\n";
	/**
	* takes help info from NeedToKnow and turns it into HTML form
	*
	*/
		static String bold_start = "<strong>";
		static String bold_end = "</strong>";

	void consolidate()
	{
		for( int i = 0; i < steps.length; ++i)
		{
			for(int j = 0; j < question_groups.length; ++j)
			{
				if ( steps[i].questionGroup_ref.equals( question_groups[j].id ) )
				{
					steps[i].wqg = question_groups[j].dup();
					
				}
			}
			
		}
	}
	static String BOLD (String string)
	{
		String out = "";
		out += bold_start;
		out += string;
		out += bold_end;
		return out;
	}

	String multi_line(String s)
	{
		String acc = "";
		String[] words = s.split(" ");
		int width = 40;

		int col = 0;
		for(int i = 0; i < words.length; ++i)
		{
			int len = words[i].length();
			if ( len < width )
			{
				if (  (col + len) < width )
				{
				}
				else
				{
					acc += "<br />";
					col = 0;
				}
				acc += words[i] + " ";
				col += (len + 1);
			}
			else // word too big - but chcuk it out anyway
			{
				acc += words[i] + "<br />";
			}
		}

	/**
		int nlines =  (s.length() + 39)/ 40 ;
		String acc = "";
		for(int i=0;i<nlines;++i)
		{
			int start = i * 40;
			int end   = (i + 1) * 40;
			if ( end > s.length() )
			{
				end = s.length();
			}
			acc += s.substring(start,end) + "<br>";
		}
             **/
		return acc;
	}
	public String to_HTML_help(int pn)
	{
		
		stage = pn;
		String contents = "";
	
		contents     = pre_amble;
		contents 	 +=	"<h1>Help</h1>\n";
		contents     += "<h2>"+ steps[stage - 1].title + "</h2>";
		//contents   += steps[stage - 1].guidance  + "<br>\n";
		contents     += "<p>"+ multi_line(steps[stage - 1].guidance) + "</p>\n";

        
		contents     +="<dl>";
		if ( ! steps[stage - 1].structure_prompt.equals("")  )
		{
			contents 	 += "<dt>";
			contents     += steps[stage - 1].structure_prompt + "</dt>\n";	// BOLD
			//contents   += steps[stage - 1].structure_guidance + "<br>\n";
			contents     += "<dd>"+ multi_line( steps[stage - 1].structure_guidance) + "</dd>\n";
		}

		int count = question_groups[0].questions.length;
		for(int i = 0; i < count; ++i)
		{
			contents 	 += "<dt>";
			contents     += question_groups[0].questions[i].prompt + "</dt>\n";	// BOLD
			//contents   += question_groups[0].questions[i].guidance + "<br>\n";
			contents     += "<dd>"+ multi_line( question_groups[0].questions[i].guidance) + "</dd>\n";
		}
		contents     +="</dl>";
		contents += post_amble;
		return contents;
	}
	/**
	* takes progress info from NeedToKnow and turns it into HTML form
	*
	*/
	public String toHTML(int stage)
	{
		String acc = pre_amble;
		String path_to_file = WckerSystem.getResourceDirectory();
		
		//acc +="\n<style>body{background:#fff url(\""+ path_to_file + "icons/progress_backgound.png\");margin:0px;font-family:verdana,sans-serif;font-size:0.9em;}ol{margin-left:1em;,font-size:0.9em;}li {margin-left:1em;padding-top:0.2em;padding-bottom:0.2em;}h1{background-color:#000080;color:#CED5EC;font-size:0.9em;padding-bottom:0.3em;padding-left:1em;} .selected{font-weight:bold;}</style>";
        //acc += "\n<style>\nbody{\nbackground:url(\"file:///D|/icons/progress_background.png\") no-repeat;\nmargin:0px;\nfont-family:verdana,sans-serif;\nfont-size:0.9em;\n}\nol\n{\nmargin-left:2em;\nfont-size:0.9em;\n}\nli \n{margin-left:1em;\npadding-top:0.2em;\npadding-bottom:0.2em;\n}\nh1\n{\nbackground-color:#000080;\ncolor:#fff;\nfont-size:0.9em;\npadding-bottom:0.3em;\npadding-left:1em;\n} .selected{font-weight:bold;}</style>";
		acc += "<h1>Progress</h1>\n";
		acc += "<ol style=\"margin-left:1em;\">";
		for(int i=1;i<=steps.length;++i)
		{
			acc += "<li ";
			if ( i == stage )
				acc += "class=\"selected\"";
			else
				acc +="";
			
			acc +=">" +steps[i-1].title + "</li>";
		}
		acc +="</ol>";
		
		acc +=post_amble;

		return acc;
	}
	/**
	* a dump of everything in a NeedToKnow object
	*
	*/
	public String toString()
	{
		String acc = "";

	if ( debug )
	{
		acc += "needtoknow start ====================++++++ |" + title + "|";
		acc +=  title + "\n";
		for(int i=0;i<steps.length;++i)
		{
			acc += steps[i] + "\n";
		}
		acc += ".....................................\n";
		for(int i=0;i<steps.length;++i)
		{
			acc += steps[i] + "\n";
			acc += ".....................................\n";
		}
		/////////acc += steps[stage];
		acc += structure_terms;
		acc += new_page_template;
		acc += organization;

		for(int i=0;i<question_groups.length;++i)
		{
			acc += question_groups[i];
		}
		acc += "needtoknow stop ====================++++++ |" + title + "|";
	}
		return acc;
	}
	/**
	* creates a NeedToKnow object from a Document object
	*/
	WckerQuestionGroup step_to_question_group(int step)
	{
/******************
		WckerStep s;

		int index = step - 1;
		if ( ( index >= 0 ) && ( index < steps.length ) )
		{
			s = steps[index];
		}
		else
		{
			s = null;
		}
		System.out.println("YYY + STEP " +  " steps.length =   " +  steps.length + "index = " + index );
		String question_group_name =  s.questionGroup_ref; 
		// need to convert name to WckerQuestionGroup
		System.out.println("YYY + " + "|" + s + "|" + question_group_name + "| question_groups.length = ");

		for (int i = 0; i < question_groups.length; ++ i )
		{
			if ( question_group_name.equals( question_groups[i].id ) )
			{
System.out.println("WANTED = |" + question_group_name + "| FOUND = |" + question_groups[i].id + "|" );
				return question_groups[0];
			}
		}

		return question_groups[0];
************/
		WckerStep s;
		int index = step - 1;
		if ( ( index >= 0 ) && ( index < steps.length ) )
		{
			s = steps[index];
		}
		else
		{
			s = null;
		}
		return s.wqg;
		
	}
	public NeedToKnow(String wizard_file_name,Document d)
	{
		filename = wizard_file_name;
		

		title = LoadWizard.document_to_title(d);
		description = LoadWizard.document_to_description(d);
		structure_terms =  LoadWizard.document_to_tree(d);
		new_page_template = LoadWizard.document_to_template(d);
		//TODO get supportFiles data filled in
		if(debug) {
			System.out.println("~##~ Get supportFiles 1 ~##~");
		}
		supportFiles = LoadWizard.support_files_from_document(d);
		organization    =  LoadWizard.document_to_tree2(d);
		steps = LoadWizard.document_to_steps(d);
		question_groups = LoadWizard.document_to_question_groups(d);
		consolidate();
/**/
	}
	public NeedToKnow(File wizardWorkingDir) {
		_initialDir = wizardWorkingDir;
		String wizard_file_name = wizardWorkingDir.toString() + WckerSystem.fileSeparator() + "wizard.wsi";
		
		if ( debug )
		{
			System.out.println("CALLING LoadXML.loadXml wizard_file_name = |" + wizard_file_name +"|");
		}

		Document d = LoadXML.loadXml( wizard_file_name );
		if (debug) {
			Navigate.RECURSIVE_print(d); // for debugging
		}

		if (debug )
		{
			System.out.print("CALLING NeedToKnow d = " + d);
		}
		
		
		filename = wizard_file_name;
		title = LoadWizard.document_to_title(d);
		description = LoadWizard.document_to_description(d);
		structure_terms =  LoadWizard.document_to_tree(d);
		new_page_template = LoadWizard.document_to_template(d);
		//TODO get supportFiles data filled in
		if(debug) {
			System.out.println("~##~ Get supportFiles 2 ~##~");
		}
		supportFiles = LoadWizard.support_files_from_document(d);
		organization    =  LoadWizard.document_to_tree2(d);
		steps = LoadWizard.document_to_steps(d);
		question_groups = LoadWizard.document_to_question_groups(d);
		consolidate();
						

	}
	NeedToKnow(Document d)
	{
		
		title = LoadWizard.document_to_title(d);
		description = LoadWizard.document_to_description(d);
		structure_terms =  LoadWizard.document_to_tree(d);
		new_page_template = LoadWizard.document_to_template(d);
		//TODO get supportFiles data filled in
		if(debug) {
			System.out.println("~##~ Get supportFiles 3 ~##~");
		}
		supportFiles = LoadWizard.support_files_from_document(d);
		organization    =  LoadWizard.document_to_tree2(d);
		steps = LoadWizard.document_to_steps(d);
		question_groups = LoadWizard.document_to_question_groups(d);
		consolidate();
		
	}
	/**
	 * Return the Directory that the wizard's course was started in.
	 * @return
	 */
	public File getInitialDir() {
		return _initialDir;
	}
	
	/**
	 * Return the Directory within which the wizard's course is being worked on
	 * Regardless of whether the user has saved the wizard in progress to a
	 * Separate directory.
	 * @return
	 */
	public File getOperatingDir() {
		if (debug) {
			System.out.println("getOperatingDir():");
			System.out.println("_workingDir: " + _workingDir);
			System.out.println("_initialDir: " + _initialDir);
		}
		File theDir;
		if (_workingDir != null) {
			theDir = _workingDir;
		} else {
			theDir = _initialDir;
		}
		return theDir;
	}
	
	/**
	 * Set the Directory within which the wizard's course is being worked on
	 * @return
	 */
	public void setWorkingDir(File newWorkingDir) {
		_workingDir = newWorkingDir;
	}

	void select_and_see( TreeItem ti)
	{
		gui_tree.setSelection(new TreeItem[] { ti } );
		gui_tree.showSelection(); 
		tbs.update_title_from_tree_item( this,  ti);
		tbs.update_6_buttons( this, ti);
	}

	static String [][] get_structure_paths(WckerTree wt, int depth)
	{
		String [][] r;
		String name = wt.getName();
		if ( wt.children.length == 0 )
		{
			//String[] level1 = { name };
			//String[][] level2 = { level1 };
			String[][] level3 = { { name } };
			return level3;
		}

		String [][][] results = new String [wt.children.length][][];
		String [][][] inputs  = new String [wt.children.length][][];
		for(int i =0; i < wt.children.length; ++i)
		{
			inputs[i]  = get_structure_paths(wt.children[i], depth + 1);
		}

		if ( depth == 0 )
		{
			for(int i =0; i < wt.children.length; ++i)
			{
				results[i] = 	inputs[i];
			}
		}
		else
		{
			for(int i =0; i < wt.children.length; ++i)
			{
				results[i] = 	WckerUtility.map_prepend( name, inputs[i] );
			}
		}

		/*
		Object[][][] 	input_cast =  ( Object[][][] ) results 
		Object[][]   	output =   f_3D_to_2D ( before_cast );
		String[][]	output_cast = (String [][])  output;
		return output;
		*/
		return WckerUtility.string_3D_to_2D(results);



	}
	static String [][] get_organisation_paths(WckerTree wt, int depth)
	{
		String name =   wt.getType() ; // nihilist
		if ( wt.children.length == 0 )
		{
			return new String[][] { { name } };
		}

		String [][][] results = new String [wt.children.length][][];

		for(int i =0; i < wt.children.length; ++i)
		{
			results[i] = 	WckerUtility.map_prepend
					( 
						name, 
						get_organisation_paths(wt.children[i], depth + 1)
					);
		}
	
		return WckerUtility.string_3D_to_2D(results);

	}
	public String[][] get_structure_paths_top()
	{
		return get_structure_paths(structure_terms,0);
	}
}
 