package uk.ac.ox.tall.wcker;

import uk.ac.ox.tall.wcker.OrganisationPayLoad;

class NodeInfo
{
	static boolean debug = WckerSystem.doDebug();
	boolean wt_version;
	NeedToKnow local_ntn;
	WckerTree wt;
	MyTreeItem ti;
	
	NodeInfo(NeedToKnow ntn, WckerTree wt)
	{
		this.wt = wt;
		local_ntn = ntn;
		wt_version = true;
	}
	NodeInfo(NeedToKnow ntn, MyTreeItem ti)
	{
		this.ti = ti;
		local_ntn = ntn;
		wt_version = false;
	}
	boolean matching(String[] current, String[] future)
	{
		if ( current.length >= future.length )
		{
			return false;			// future does not extend beyond present!
		}
		else
		{
			for(int i=0;i<current.length;++i)
			{
				if ( ! future[i].equals( current[i] ) )
				{
					return false;  // not a valid future 
						       // i.e. is not consistent with present
				}
			}
		}
		return true;
	}

	int matching_I(String[] current, String[] future)
	{
		int over_hang = future.length - current.length;
		if ( over_hang < 0 )
		{
			return 0;			// future does not extend beyond present!
		}
		else
		{
			for(int i=0;i<current.length;++i)
			{
				if ( ! future[i].equals( current[i] ) )
				{
					return 0;  // not a valid future 
						       // i.e. is not consistent with present
				}
			}
		}
		return over_hang;
	}

	String[] select_possible(String[] organisation_path, String[][] possibles)
	{
		String[] r = { "nope" };
		boolean[] valid = new boolean[possibles.length];
		int count = 0;
		for(int i = 0; i < possibles.length; ++i)
		{
			valid[i] = matching( organisation_path, possibles[i] );
			if ( valid[i] ) 
			{
				count++;
			}
		}
		if ( count == 0 )
		{
			String[] null_list = new String[0];
			return null_list;
		}
		int index = organisation_path.length;
		String[] words = new String[count];
		int word_count = 0;
		for (int i = 0 ; i <  possibles.length; ++i)
		{
			if ( valid[i] ) 
			{
				words[word_count] = possibles[i][index];
				word_count++;
			}
	
		}
		return WckerUtility.make_unique(words);
	}
	/* returns two lists
	 first is list of container nodes that are possible 

			possible is exactly  1 longer than actual

	second is  list of leaf nodes that are possible 
		
			possible is exactly  2 or more than actual*/ 

	String[][] select_possible2(String[] organisation_path, String[][] possibles)
	{
		String[] r = { "nope" };
		int[] valid = new int[possibles.length];
	

		int leaf_index      = 0;
		int container_index = 0;

		for(int i = 0; i < possibles.length; ++i)
		{
			valid[i] = matching_I( organisation_path, possibles[i] );
			switch( valid[i] )
			{
				case 0:		break;
				case 1: 	leaf_index++; break; // leaf count
				default:	container_index++; break; // container count
			}
		}
		String[][] words = { new String[container_index], new String[leaf_index] };
			
		int index = organisation_path.length;

		leaf_index      = 0;
		container_index = 0;
		for (int i = 0 ; i <  possibles.length; ++i)
		{
			switch ( valid[i] ) 
			{
				case 0:		break;
				case 1: 	words[1][leaf_index++] = possibles[i][index];break;
				default:	words[0][container_index++] = possibles[i][index];break; // container count		
			}
	
		}
		return new String[][]{  WckerUtility.make_unique(words[0]), WckerUtility.make_unique(words[1]) };
	}

String wckertree_to_type(WckerTree wt)
{
	OrganisationPayLoad opl = (OrganisationPayLoad) ( wt.payload );
	String type = opl.type;
	return type;

}
String treeitem_to_type( MyTreeItem ti)
{
	Object o = ti.payload;
	WckerTree wt = ((WckerTree) o);
	return wckertree_to_type(wt);
}
	String [] postfix( String[] old_sentence, String word)
	{
			int n_words =  old_sentence.length;
			String [] new_sentence = new String[ n_words + 1];
			
			
			for (int j = 0; j < n_words; ++j)
			{
				new_sentence[j] = old_sentence[j];
			}
			new_sentence[n_words] = word;
			return new_sentence;
	}

String[] get_organisation_path0(WckerTree wt)
{

	
	String type = ( (OrganisationPayLoad) wt.payload ).type;
	if ( wt.parent == null )
	{
		return new String[] { type };
	}
	else
	{
		return postfix(  get_organisation_path0( wt.parent), type );

	}
}
String[] remove_first(String[] in)
{
	int l = in.length - 1;
	String[] out = new String[l];
	for(int i=0;i<l;++i)
	{
		out[i] = in[i+1];
	}
	return out;
}
String[] get_organisation_path(WckerTree wt)
{
	 return remove_first(remove_first(get_organisation_path0( wt)));
}

String[] get_organisation_path( MyTreeItem ti)
{
	String type = treeitem_to_type( ti);
	MyTreeItem ti2 = ti;
	int index = 1;
	while ( ( ti2 = (MyTreeItem) ti2.getParentItem() ) !=  null)
	{	
		if ( debug )
		{
			System.out.println("CLEVER ALGORITHM winding up = " + treeitem_to_type( ti2 ));
		}
		index++;
	}
	int count = index;
	String[] path = new String[count];

	index = count - 1;
	path[index] = type;
	ti2 = ti;
	while ( ( ti2 = (MyTreeItem) ti2.getParentItem() ) !=  null)
	{	
		index--;
		path[index] = treeitem_to_type( ti2 );
		if ( debug )
		{
			System.out.println("CLEVER ALGORITHM winding down = " + treeitem_to_type( ti2 ));
		}
	}
	


	if ( debug )
	{
		System.out.print("CLEVER_ALGORITHM count = " + count + "path =");
		for(int i=0;i<path.length;++i)
		{
			System.out.print("\\" + path[i]);
		}
		System.out.println("");
	}

	return path;
}
	// returns "container" or "page"
	String type()
	{
		String[] organisation_path = label_path();
		//String[] organisation_path =  get_organisation_path(ti);
		String[][] possibles = local_ntn.get_structure_paths_top();
		String[] user_choice = select_possible(organisation_path, possibles);
		if ( user_choice.length == 0 )
		{
			return "page";
		}
		else
		{
			return "container";
		}
	}
	// returns "Course", "Unit" or "Page", etc
	// returns { "Course", "Unit" }, etc
	String label()
	{
		if ( wt_version )
		{
			return wckertree_to_type(wt);
		}
		else
		{
			return treeitem_to_type(ti);
		}
	}
	String[] label_path()
	{
		if ( wt_version )
		{
			return get_organisation_path(wt);
		}
		else
		{
			return get_organisation_path(ti);
		}
	}
	
	CleverReturn clever_algorithm()
	{
		boolean self_insertion_mode;
		String[] organisation_path   = label_path();
		String[][] possibles = local_ntn.get_structure_paths_top();
		String[][] user_choice = select_possible2(organisation_path, possibles);
		if ( ( user_choice[0].length == 0) && (user_choice[1].length == 0) )
		{
			// nothing valid to add means we are at leaf mode
			// add more of the same but to THE PARENT

			 user_choice =  new String[][]
					{
						new String[0], 
						new String[] {/* "PARENT " + */ organisation_path[ organisation_path.length - 1] } 
					};
			self_insertion_mode = false;
			
		}
		else
		{
			self_insertion_mode = true;
		}


		// print out for clever_algorithm

		if ( debug )
		{
			System.out.println("CLEVER ALGORITHM ORGANISATION =" + WckerUtility.string_list_to_path(organisation_path) );
			for(int i = 0; i < possibles.length; ++i)
			{
				System.out.println
				(
					"CLEVER ALGORITHM POSSIBLES[" 
					+  i  
					+  "]="  
					+ WckerUtility.string_list_to_path(possibles[i]) 
				);
			}
			for(int j=0; j < 2; ++j)
			{
				for(int i = 0; i < user_choice[j].length; ++i)
				{
					System.out.println
					(
						"CLEVER ALGORITHM CHOICES["
						+  j
						+ "][" 
						+  i  
						+  "]="  
						+ user_choice[j][i] 
					);
				}	
			}
		}
		return new CleverReturn(self_insertion_mode, user_choice);
	}
	static void main( String[] args)
	{
		System.out.println("static main method in class NodeInfo");
	}
}
