package uk.ac.ox.tall.wcker;

import org.eclipse.swt.widgets.*;
import org.eclipse.swt.graphics.*;

/**
 * WckerIcon class - handles icons for WCKER
 *                   provide a simple meaningful icon name and get an image object back
 *                   which has been read from an appropriate file.
 *		     unfortunately the Image class is final so cannot be extended
 *                   
 * @author BalchD
 * @author JohnstonD
 *  
 */

public class WckerIcon 
{
    	private static boolean debug = WckerSystem.doDebug();

	private static String[] possible_types = 
		{ "course", "unit", "page", "Course", "Unit", "Page", 
		"add_unit", "delete_unit", "add_page", "delete_page", "up", "down" , "container",
		 "course_grey", "unit_grey", "page_grey", "add_unit_grey", "delete_unit_grey", "add_page_grey", "delete_page_grey", "up_grey", "down_grey" , "container_grey" };

	private static String[] image_files    = 
		{ "course", "icon1", "icon2", "course", "icon1", "icon2", 
		"add_unit", "delete_unit", "add_page", "delete_page", "up", "down", "icon1",
		 "", "icon1_grey", "icon2_grey", "add_unit_grey", "delete_unit_grey", "add_page_grey", "delete_page_grey", "up_grey", "down_grey", "icon1_grey" };

	private static Image[] pre_load = new Image[ possible_types.length ];
	

	private static String resourceDir = WckerSystem.getResourceDirectory() + "/icons/";

	static Image node_info_to_image(NodeInfo ni)
	{
		String[] lp = ni.label_path();

if ( debug )
{
System.out.print("node_info_to_image " + WckerUtility.string_list_to_path(lp) + " | " );
}


		String name;
		if ( lp.length == 1 )
		{
			name = "Course";
		}
		else
		{
			if ( ni.type().equals( "container" ) )
				name =  "Unit";
			else
				name =  "Page";
		}
		return(  type_to_image( Display.getDefault(), name ) );
	}
	static Image my_tree_item_to_image(NeedToKnow local_ntn, MyTreeItem mti)
	{
		NodeInfo ni = new NodeInfo(local_ntn,mti);
		return node_info_to_image( ni);
		
	}
	static Image wcker_tree_to_image(NeedToKnow local_ntn, WckerTree wt)
	{
		NodeInfo ni = new  NodeInfo(local_ntn,wt);
		return node_info_to_image( ni);
	}
	
	private String type_to_file_name(String type)
	{
		String match = "";
		String path = "";
		
		for(int i = 0; i < possible_types.length ; ++i)
		{
			if  ( type.equals(possible_types[i]) )
			{
				match = resourceDir + image_files[i] + ".png";
				
			}
		}
		return match;
	}
	/**
	* converts icon name to Image
	*
	* @param d display on which we are operating
	* @param type name of icon
	* @return Image object holding picture of icon
	*/
	public static Image type_to_image(Display d, String type)
	{
		return type_to_image(d,type, false);
	}
	public static Image type_to_image(Display d, String type, boolean greyed_out)
	{
		int index = -1;
		String name = "";

		if (debug)
		{
			System.out.println("type_to_image " + 			 type);
		}
	
		
		for(int i = 0; i < possible_types.length ; ++i)
		{
			if  ( type.equals(possible_types[i]) )
			{
				index = i;
				break;
				
			}
		}
		if ( index > 0)
		{
			if ( pre_load[index] == null )	// FILE IS NOT PRELOADED
			{
				//greying out is automatically handled but SWT button
				//name = resourceDir + image_files[index] + (  greyed_out ? "_grey" : "") + ".png";
				name = resourceDir + image_files[index] + ".png";
				try
				{
					pre_load[index] = new Image(d , name);
					if ( debug )
					{
						System.out.println("LOADING IMAGE" + 			name);

					}
					return new Image(d , name);	// LOAD FILE
				}
				catch (Exception e)
				{
					return (Image) null;
				}
			}
			return pre_load[index];
		}
		return (Image) null; // CANNOT IDNETIFY WHICH FILE TO LOAD!
	}	

}
