/*
 * Created on 14-Mar-2005
 *
 * @author David Balch david.balch@conted.ox.ac.uk
 * 
 */
package uk.ac.ox.tall.wcker;

import java.io.File;
import java.io.FilenameFilter;
import java.io.IOException;
import java.text.Format;
import java.text.SimpleDateFormat;
import java.util.Date;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;

import uk.ac.reload.diva.util.ZipUtils;

/**
 * A class to deal with loading data from WCKER Wizard packages (.wwz)
 * @author David Balch david.balch@conted.ox.ac.uk
 *
 */
public class WizPackageHandler {
	private static boolean debug = WckerSystem.doDebug();

	/**
	 * Get a list of the wizard (.wwz) files in the wizard directory.
	 * @return File[]
	 */
	public static File[] getWizardFiles() {
		//Create File array
		File WizardDirectory = new File(WckerSystem.getWizardDirectory());
		
		// Filter the file array - only include .wwz files.
		File[] WizardFiles = WizardDirectory.listFiles(
				new FilenameFilter() {
					public boolean accept(File WizardDirectoryFiles, String name) {
						return name.endsWith(".wwz");
					}
				});
		
		return WizardFiles;
	}
	
	public static WizardCacheItem CacheItemFromPackage(File newPackage) throws IOException {
		WizardCacheItem item = null;

		String file = newPackage.getName();
		Document wsiDoc;
		String title = null;
		String description = null;
		String version = null;
		
		if (debug) {
			System.out.println("");
			System.out.println("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
			System.out.println("Adding wizard package to cache: " + newPackage.getName());
		}
		
		/* Doesn't work
			// TODO: Make this work
			// Extract the wizard.wsi into memory and create a Document object directly
			// from the zip file. I.e. don't load from disk, save to disk, load from disk.
			final int bufSize = 1024;
			
			// The .wsi file in the package zipfile
			ZipFile zipFile = new ZipFile(newPackage);
			ZipEntry zipWsi = zipFile.getEntry("wizard.wsi");
			int zipWsiSize = (int)zipWsi.getSize();
			
			String ZipEntryString = ZipUtils.extractZipEntry(newPackage, "wizard.wsi");
			
			InputStream wsiInputStream = zipFile.getInputStream(zipWsi);
			
			
			//InputStream wsiInputStream = ZipUtils.extractZipEntryStream(newPackage, "wizard.wsi");
			
			if (debug) {
				//System.out.println("ZipEntryString: " + ZipEntryString);
				System.out.println("zip: " + zipFile.getName());
				System.out.println("    zipWsi.getName(): " + zipWsi.getName());
				System.out.println("    zipWsiSize: " + zipWsiSize);
				System.out.println("    wsiInputStream: " + wsiInputStream);
				
				byte[] wsiInputStreamData = new byte[zipWsiSize];
				int n_read = wsiInputStream.read(wsiInputStreamData, 0, zipWsiSize);
				System.out.println("    Data node: " + wsiInputStreamData);
				System.out.println("    Bytes read from wsiInputStream (n_read): " + n_read);
				
				String someString = new String(wsiInputStreamData);
				System.out.println("someString: " + someString);
				
				 InputStreamReader contents = new InputStreamReader(wsiInputStream);
				 byte[] wsiInputStreamReaderData = new byte[zipWsiSize];
				 int n_read_reader = wsiInputStream.read(wsiInputStreamReaderData, 0, zipWsiSize);
				 System.out.println("    data node: " + wsiInputStreamReaderData);
				 System.out.println("    Bytes read from wsiInputStreamReaderData (n_read_reader): " + n_read_reader);
				 
				 String someString2 = new String(wsiInputStreamReaderData);
				 System.out.println("someString: " + someString2);
				 
			}
			
			//load the .wsi file
			wsiDoc = LoadXML.loadXml(wsiInputStream);
			
			if (debug) {
				System.out.println("wsiInputStream: " + wsiInputStream);
				System.out.println("wsiXml: " + wsiDoc);
			}
			
			*/
		
		File tempFile = new File(WckerSystem.getTempDir() + "newWizTmp.xml");
		// Ensure that the parent Folder exists
		if(!tempFile.getParentFile().exists()) {
			tempFile.getParentFile().mkdirs();
		}
		
		File wsiFile = ZipUtils.extractZipEntry(newPackage, "wizard.wsi", tempFile);
		
		//load the .wsi file as an XML Document
		wsiDoc = LoadXML.loadXml(wsiFile);
		
		//Setup XPath stuff
		XPathFactory xpf = XPathFactory.newInstance();
		XPath xpath= xpf.newXPath();
		
		try {
			version = xpath.evaluate("/wizard/metadata/version", wsiDoc.getDocumentElement());
			title = xpath.evaluate("/wizard/metadata/title", wsiDoc.getDocumentElement());
			description = xpath.evaluate("/wizard/metadata/description", wsiDoc.getDocumentElement());
			
		} catch (XPathExpressionException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		if (debug) {
			System.out.println("");
			System.out.println("Adding details from package: " + newPackage);
			System.out.println("  Title:       '" + title + "'");
			System.out.println("  Description: '" + description + "'");
			System.out.println("  Version:     '" + version + "'");
			System.out.println("  File:        '" + file + "'");
		}
		
		item = new WizardCacheItem(title, description, file, version);
		return item;
	}

	/**
	 * Unzip a .wwz wizard package to the working directory
	 * @return File
	 */
	public static File extract_package(String wizardPackage) {
		
		// The package to be unzipped
		File packageFile = new File(WckerSystem.getWizardDirectory() + wizardPackage) ;
		
		// The output directory name, based on the wizard package name and the current datetime
		
		// Remove the ".wwz" file extension
		String wizardName = wizardPackage.substring(0, wizardPackage.lastIndexOf(".wwz"));
		
		// Get the current datetime, and format it.
		Date date = new Date();
		Format formatter = new SimpleDateFormat("yyyyMMddHHmmss");
		String dateString = formatter.format(date);
		
		// Concatenate into a path
		File targetFolder = new File(
				WckerSystem.getWorkingDirectory() + 
				wizardName + 
				"_" + 
				dateString
		);
		
		if (debug) {
			System.out.println("    wizardName: " + wizardName);
			System.out.println("  targetFolder: " + targetFolder);
		}
		
		try {
			// Use the ZipUtils in Reload Diva to unpack the wizard package    	
			ZipUtils.unpackZip(packageFile, targetFolder);
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			if (debug) {
				System.out.println("Couldn't unzip wizard package: " + wizardPackage);
			}
		}
		return targetFolder;
	}
}
